#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2011-2014 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'prawn'
require 'prawn/table'
require 'escape'
require_relative 'logging'
require_relative 'configuration'
require_relative 'translating'
require_relative 'html2text'

# NS_POS=1
# TAG_POS=2
# CODE_POS=0


# An object of this class wrapps the Prawn PDF generator to convert a single
# message from an eml-file to PDF.
class CFPrawn
	include Prawn
	include Translating
	include Logging

	@@DEF_LINK_COLOR='008000'
	@@DEF_REPLACEMENT_COLOR='808080'
	@@DEF_CONTENT_TYPE_COLOR='000080'
	@@DEF_REPLACEMENT="<?>"
	public
	# initialize a CFPrawn instance, prepare the tag-handlers
	# for the processing of HTML-mail and begin splitting up the
	# message into body, headers, attachments.
	# Treat all of that some way or other to create a PDF-document.
	def initialize(mail, path) 
		init_logger($CONF.log_file ? $CONF.log_file : STDOUT, $CONF.log_level ? $CONF.log_level : Logger::UNKNOWN)
		pfx = $CONF.pdf_prefix
		sfx = $CONF.pdf_suffix
		page_size = $CONF.page_size
		page_size ||= 'A4'

		page_layout = $CONF.page_layout
		page_layout ||= 'portrait'
		
		@msg = nil		
		@doc = Document.new(:page_size => page_size, :page_layout => page_layout.to_sym)
		@parts_count = 0
		@headers = mail.headers ? mail.headers : []
		begin
			@log.debug(trl("headers are %s") %[@headers.join(', ')])
	        rescue Exception => e	
			@log.warn(trl("Found a potential problem with the mail-headers.") << " " << e.message)
		end
		@body = mail.body
		@attachments = mail.attachments
		@attachment_option = $CONF.save_attachments
		@attachment_option ||= false
		check_fonts	
		render
		@outfile = path.dup << File::Separator << (pfx ? pfx : '') << File::basename(mail.filename) << (sfx ? sfx : '') << '.pdf'
		@log.debug(Translating::trl("writing file %s") %outfile )

		if(@attachment_option && !@attachments.empty? )
			if('pdf' == @attachment_option)
				# create PDF, attach files
				@doc.render_file(@outfile)
				handle_attachments(@outfile)
			else
				# create attachment-files, link, then create PDF
				handle_attachments(@outfile)
				@doc.render_file(@outfile)
			end
		else
			# create directly PDF, ignore attachments.
			@doc.render_file(@outfile)
		end
	end
	# Finds font-files and defines the font-settings for the
	# PDF.
	def check_fonts()
	
		user_font = {}
	        # ***BUGFIX*** Story: if one font-file does not exist, but the other does,
		# use both anyway and crash.
		# READ TWICE: I AM TOO DUMB TO PROGRAM !	
		msg = trl('user-defined fonts cannot be applied please verify your configuration.')

		{:normal=>$CONF.regular_font, :bold=>$CONF.bold_font, :italic=>$CONF.italic_font, :bold_itlic=>$CONF.bold_italic_font}.each_pair do |fs, font|
			if(font && !font.strip.empty?)
				if(File.exist?(font ))
					user_font[fs] = font	
				else
					msg << " (" << trl("font-file '%s' does not exist") %(font ? font.to_s : ' NIL ') << ")"
					@log.warn(msg)
					# do not use *ANY* user-defined font
					return
				end
			else 
				@log.warn(msg)
				return
			end
		end
		@doc.font_families.update("user_font" => user_font) if !user_font.empty?	
		@doc.font("user_font") if !user_font.empty?
		@log.debug("using fonts: " << @doc.font_families.inspect)
	end
	# Delegates method-calls to Prawn, or not.
	def method_missing(m, *args)
		@log.debug('method missing ' << m.to_s << '( ' << args.join(', ') << ')')
		if(@doc.respond_to?(m) )
			@doc.send(m, *args)
		else 
			@msg = trl("The method %s is not defined for objects of type %s") %[m, self.class.name] 
			@log.error(@msg)
		end
	end

	private
	# handles attachment-files in the way, that is configured by the users.
	def handle_attachments(outfile) 
		new_out = outfile.dup << '_attachment'
		attachment_files = []

		@log.debug('@attachment_option is ' << @attachment_option.to_s)

		if('pdf' == @attachment_option )
			if($Progs['pdftk'])
				dir = File.dirname(outfile) + File::Separator
				fl = ""
				@attachments.each do |at|			
					fl = dir.dup << at.filename
					@log.debug('writing attachment-file ' << fl)
					File.write(fl, at.decoded)
					# add quote-marks to file-names.
					attachment_files << ("%s" %fl )
				end

				# cmd = Escape.shell_command(["pdftk", pdf, "update_info", info_file, "output", t_file]).to_s
				cmd = Escape.shell_command(["pdftk", outfile, "attach_files", *attachment_files, "output", new_out]).to_s
				# cmd = "pdftk \"#{outfile}\" attach_files #{attachment_files.join(' ' )} output \"#{new_out}\""	

				@log.debug('executing ' << cmd)
				if(! system cmd)
					@msg = trl("Failed to attach files with pdftk (%s)." %($?.exitstatus) )  
					@log.error(@msg)
				end
				begin
					# remove quote-marks from the file-name prior deletion.
					attachment_files.each {|f| File.delete(f.gsub("\"", '') ) }	
				rescue Exception => ex
					@msg = trl("Trying to remove an attachment file resulted in an error (%s)") %ex.message 
					@log.error(@msg)
				end
				if($?.success? && File.exist?(new_out) )
					begin
						File.unlink(outfile)
					rescue Exception => ex
						@msg = trl("Trying to remove a file resulted in an error (%s)") %ex.message 
						@log.error(@msg)
					end
					File.rename(new_out, outfile)
				else
					@log.warn(trl("Trying to attach files to %s, the call to pdftk returned with exit-status %s") %[outfile, $?.exitstatus])
				end
			else 
				@log.warn(trl("PDF attachments are requested, but the pdftk-utility has not been found! Ignoring the attachments."))
			end
		else 
			if(File.exist?(@attachment_option) && File.directory?(@attachment_option) )
				fc = 0
				@attachments.each do |at|			
					@log.debug('handling attachment: ' << at.filename)
					@log.debug(at[:content_type])
					ctype = ''
					begin
						ctype = at[:content_type].to_s.split('/')[1].split(';')[0]
					rescue Exception=>ex
						@log.warn("a problem was encounterd in analyzing the content-type of an attachment: " << ex.message)
					end
					@log.debug('attachment, content-type: ' << ctype) if ctype
					fl = @attachment_option.dup << File::Separator << at.filename
					File.write(fl, at.decoded)
					if(File.exist?(fl) )	
						fc += 1
						if(fc == 1)
							text(' ')
							text("<b>" << trl("Attached files") << ":</b>" , :inline_format=>true)
						end
						link_color = $CONF.attachment_link_color
						link_color ||= @@DEF_LINK_COLOR
						@log.debug('link color is ' << '#' << link_color.to_str)
						text(format("%i) %s: <link href='file://%s'><u><color rgb='%s'>%s</color></u></link>", fc, ctype, fl, '#' << link_color, at.filename), :inline_format=>true)
					end
				end
				@log.info(trl("Have saved %s of %s attachments to %s") %[fc, @attachments.size, @attachment_option] )
			else
				@msg = trl("The value of option 'save attachments' is not a valid directory-name (%s)!" ) %[@attachment_option] 
				@log.error(@msg)
			end
		end
	end

	# handles mail-headers and the mail-body
	def render()
		if (@headers && !@headers.empty? )
			@log.debug('writing headers')
			write_headers()
		end

		if (@body && !@body.empty?)
			body_parts(@body)
		end
	end
	# handles one 'part' of a message. Will be
	# called for each 'part' of a multipart-message.
	def write_part(btext, content_type = nil)
		move_down(20)
		if(content_type)
			if(@parts_count >= 1)
				start_new_page
			end
			if($CONF.show_content_type) 
				ct_color = $CONF.content_type_color
				@log.debug('read content-type-color from config ' << ct_color) if ct_color
				ct_color ||= @@DEF_CONTENT_TYPE_COLOR
				@log.debug('using content-type-color ' << ct_color)
				text(format("<u><b><color rgb='%s'>%s</color></b></u>", '#' << ct_color, content_type), :inline_format=>true)
				move_down(5)
			end
		end
		begin
			@log.debug('btext is ' << btext)
			text(btext.encode!("UTF-8"), :inline_format=>true)
		rescue Exception=>ex
			@log.warn(ex.message)	
			nt = ''
			replacement = $CONF.replacement_char
			replacement ||= @@DEF_REPLACEMENT
			replacement.gsub!('<', '&lt;')
			if(btext.respond_to?(:each_char))
				btext.each_char do |c|
					begin
						# a last hope
						nt << c.encode('ASCII').encode('UTF-8')		
					rescue Exception => ex
						# now I give up.
						col = $CONF.replacement_color
						if(! col || col.size != 6)
							#col = '808080'
							col = @@DEF_REPLACEMENT_COLOR
						end
						nt << format("<color rgb='%s'>%s</color>", '#' << col, replacement)
						@log.warn(trl("encoding of a character failed (%s)!") %ex.message )
					end
				end
			end
			text(nt, :inline_format=>true)
		ensure
			@parts_count +=1
		end
	end
	# handles an email according to content-type, the fact that it
	# is a multipart-message or not. This is tricky and I am not sure
	# to have understood everything. If something goes wrong, look here, first. 
	def body_parts(part)
		content_type = part.content_type if part.respond_to?(:content_type)
		@log.debug('part_content_type is ' << content_type) if content_type
		content_type ||= @content_type
		@log.debug('part_content_type will be ' << content_type) if content_type
		# recursively handle all body-parts, let the body-text grow
		if(part.multipart? && !part.parts.empty? )	
			part.parts.each do |p| 
				body_parts(p) if !p.attachment?
			end
		elsif !part.respond_to?(:content_type) || part.content_type && part.content_type.include?('text') 
			# detect or set the charset
			# --- from the content-type of the body-part
			charset = content_type.split(';').detect {|ct| ct.include?('charset')} if content_type
			charset = charset.split('=')[1] if charset
			charset = charset.strip.gsub('"', '') if charset 
			# --- from a previously set content-header, if applicable
			charset ||= @charset 
			# --- or deduce from the encoding of the body-part.
			# @log.debug('charset is ' << (charset ? charset : 'NIL') )
			if(!charset && part.respond_to?(:encoding) )
				# @log.debug('part.encoding is ' << part.encoding)
				charset = part.encoding 
			end
			charset = nil if charset && charset.upcase == 'QUOTED-PRINTABLE'
			charset = (charset.downcase == '8bit' ? 'iso-8859-1' : charset  ) if charset
			charset = (charset.downcase == '7bit' ? 'ASCII' : charset) if charset
			# Inshallah
			btext = part.decoded
			@log.debug('decoded part is ' << btext)
			# Decoding leaves the text without an encoding (ASCII-8BIT or binary).
			# Enforce the charset, if one was found above.
			if(charset && "UTF-8" != btext.encoding.name)
				@log.debug('trying to force_encode to ' << charset.upcase)
				btext = btext.force_encoding(charset.upcase) 
			end
			@log.debug('btext-encoding is now ' << btext.encoding.to_s)
			# this should not be needed.
			replacement = $CONF.replacement_char
			@log.debug('replacement is: ' << replacement)
			if !replacement
				replacement = @@DEF_REPLACEMENT
			end
			@log.debug('charset will be ' << (charset ? charset : 'NIL') )
			# if charset
			#	btext.encode!(charset.upcase, :undef=>:replace, :invalid=>:replace, :replace=>replacement) 
			#	end

			if(content_type && content_type.include?("text/html"))
				# replace text by a modified version
				htext = html(btext)
				write_part( htext, content_type)
			else
				write_part(btext, content_type)
			end
		end
	end

	# handles html-text
	def html(html)
		parser = Html2Text.new(html)
		phtml = parser.text()
		return phtml
	end
	# extracts those headers that should appear in the final PDF-document 
	# and includes them for rendering in the PDF.
	def write_headers()
		encs = %w"UTF-8 utf-8 ISO-8859-1 iso-8859-1 ISO-8859-15 iso-8859-15"
		hcount = @headers.size
		vh = @headers
		content_type_header = @headers.detect {|h| h.respond_to?(:name) && "Content-Type" == h.name}
		@content_type = content_type_header.value if content_type_header
		cset = (content_type_header.value.split(';').detect {|ct| ct.include?('charset' ) } ) if content_type_header
		cset = cset.split('=')[1] if cset
		@charset = cset.strip.gsub('"', '').upcase if cset 
		# @log.debug('have charset from header: ' << @charset) if @charset
		if($CONF && $CONF.visible_headers)
			vh = @headers.partition {|h| $CONF.visible_headers.include?(h.name)}[0]
			hcount = vh.size
		end
		table_data = []
		hwidth = 300
		headr = vh.max do |a,b| 
			if(a.respond_to?(:name) && b.respond_to?(:name)) 
				a.name.length <=> b.name.length 
			end
		end
		if(headr)
			length_field = headr.name.dup
			@doc.font.normalize_encoding!(length_field)
			hwidth = @doc.width_of(length_field) + 30
		end
		@log.debug('prior mangling headers, charset is: ' << (@charset ? @charset : 'N I L') )
		vh.each do |h| 
			hvalue = h.decoded.dup
			hvalue.encode!('UTF-8', :undef=>:replace, :invalid=>:replace, :replace=>"<?>") 
			@log.debug('UTF-8 header: ' << hvalue)
			table_data << ["%s: " %h.name, hvalue] if h.name && hvalue		
		end
		@log.debug("printed headers are: " << table_data.join("\n"))
		if(table_data && !table_data.empty?)
			# TODO: flexibilize with configuration
			header_table = @doc.make_table(table_data, :cell_style=>{:borders=>[], :inline_format=>false}, :column_widths=>{0=>hwidth})
			header_table.columns(2).font_style = :bold
			header_table.draw
		else
			@msg = trl('Cannot write mail-headers. Are you sure that this is an EML-file?' ) 
			@log.error(@msg)
		end
	end
	public
	attr_reader :outfile, :headers, :msg
end

