#!/usr/bin/env ruby 
#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2011-2014 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'yaml'
require_relative "translating"
require_relative "logging"
require_relative "configuration"


class ConfCheck
	include Translating
	include Logging

	@@RD = File.expand_path(File.dirname(__FILE__) ) +  File::SEPARATOR if !defined?(@@RD)

	def initialize
		init_logger($CONF.log_file ? $CONF.log_file : STDOUT, $CONF.log_level ? $CONF.log_level : Logger::UNKNOWN)

		@orig_conf = "#{@@RD}config"
		if $CONF.has_user_conf
			@user_conf = $CONF.config_file if @log.level = Logger::DEBUG
			begin
				@orig_keys = YAML::load_file(@orig_conf).keys
			rescue Exception => ex
				@log.warn(trl("Cannot load the configuration-file %s") %(@orig_conf) << " (" << ex.message << ")");
			end
			@user_keys = $CONF.keys
		else
			@user_conf = $CONF.user_configuration
			# @log.warn(trl("There is no user-configuration found in the current user's home-directory. Did you expect something else?") )
		end
	end

	# Return the differences; by default those that indicate the changes
	# needed in the user-configuration so that it contains the same keys
	# as the 'global' or 'original' version of the configuration.
	def diffs(conf = :user)
		test_keys = (conf == :user ? @user_keys : @orig_keys)
		comp_keys = (conf == :user ? @orig_keys : @user_keys)
		if(test_keys && comp_keys)
			return {:missing => comp_keys - test_keys, :additional => test_keys - comp_keys}
		end
		return nil
	end

	# Make a backup-copy of the current user-coniguration.
	# Overwrite the current user-configuration with a copy of the current global/original 
	# configuration-file or create a new user-configuration.
	def write_user_conf
		if(@user_conf && File.exist?(@user_conf))
			backup = @user_conf.dup << '_bak'
			File.open backup, 'w' do |bc|
				File.open @user_conf, 'r' do |uc|
					while uc.gets("\n") do 
						bc << $_
					end
				end
			end
		end

		copy_file @orig_conf, @user_conf
	end

	private

	# dumbly copy one textfile "ifl" into the other "ofl"
	def copy_file(ifl, ofl)
		begin
			File.open ifl, 'r' do |inf|
				@log.debug('opened for reading ' << ifl)
				begin 
					File.open ofl, 'w' do |outf|
						@log.debug('opened for writing ' << ofl)
						while inf.gets("\n") do 
							outf << $_ 
						end
					end			
				rescue Exception => ex
					@log.error(trl("Cannot open file for writing (%s)") %ex.message ) 
				end
			end
		rescue Exception => ex
			@log.error(trl("Cannot open file for reading (%s)") %ex.message ) 
		end

	end
end

##### TEST #####
if __FILE__ == $0
	cf = ConfCheck.new
	cf.write_user_conf
end
