#!/usr/bin/ruby
#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2011-2014 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'rubygems'
require_relative "confcheck"
require_relative "translating"
require_relative "logging"
require_relative "configuration"
require_relative "version"

['prawn-table', 'mail', 'prawn', 'nokogiri', 'escape'].each do |g|
	begin
		gem g
	rescue LoadError
		puts '   =>   ' << (Translating::trl("PSE wait, installing required gem '%s'") %g)
		puts '        ' << Translating::trl('This may take some time') << ' ...'
		begin	
			if  !system 'gem install ' << g
				exit false
			end
		rescue Exception => e
			puts '        ' << (Translating::trl("Cannot install '%s'") %g) << ': ' << e.message
			exit false
		end
	end
	Gem.clear_paths
end
require_relative "emlfile"
require_relative "cfprawn"
require_relative "busy_indicator/busy_indicator"
require 'escape'

$PROG_DIR = File::expand_path(File::dirname(__FILE__) ) << File::Separator if !defined?($PROG_DIR)
$ICON_DIR = $PROG_DIR.dup << 'icon' << File::Separator if !defined?($ICON_DIR)
@log.debug("PROG_DIR is " << $PROG_DIR) if @log
$WIN = (RUBY_PLATFORM =~ /mswin/)


# The main application-class.
class CremeFraiche
	include Translating
	include Logging

	@@Info_Key = 'InfoKey: '
	@@Info_Value = 'InfoValue: '
	# instantiate an object of type CremeFraiche and define the
	# versioning-information. Then does the rest... You need to
	# provide the mail-files to a call of this method.
	def initialize(*args)
		init_logger($CONF.log_file ? $CONF.log_file : STDOUT, $CONF.log_level ? $CONF.log_level : Logger::UNKNOWN)
		@app_name = 'Creme Fraiche'
		@version = VERSION
		@years = YEARS
		@author = 'Michael Uplawski'
		@author_mail =  '<michael.uplawski@uplawski.eu>'
		@file_path = __FILE__
		@web_page = 'http://www.rubygems.org/gems/cremefraiche'
		@msg = nil
		@license = "This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version."     
		if RUBY_PLATFORM.include?('linux')
			@cur_system =  `uname -a` 
		else
			@cur_system = RUBY_PLATFORM
		end
		@log.debug('args.flatten.empty? ' << args.flatten.empty?.to_s)
		tfile = nil
		if(args && args.flatten.empty? )
			@log.debug('printing usage-message')
			puts usage
		elsif args[0] && 'user-conf' == args[0].chomp
			@log.debug('handling user-configuration')
			handle_user_conf
		else 
			if args.length == 1 && args[0] == '-'
				args.compact!
				args.shift
				# @log.debug('file piped-in: ' << $<.read)
				# a file may be piped in from the command-line...
				if(!$<.eof?)	
					# ... provide a temporary file in that case,
					tfile = "%s/mail_message" %(Dir.pwd)
					File.open(tfile, 'w+') {|of| of << $<.read}
					@log.debug('Temporary eml-file is written.')
					# ... then do as if it had been part of the argument-list.
					args << tfile
					# And I don't care. Really. Thanks.
				end
			end
			@log.debug('processing ' << args.size.to_s << ' files')
			# check_programs_installed
			thr = BusyIndicator.new(true, 10)
			args.each do |a| 
				num_files = 0
				mailfile = EmlFile.new(a)
				if(!mailfile.msg || mailfile.msg.empty?)
					num_files = mailfile.files.size	if mailfile.files
					mailfile.mails do |m| 
						output = CFPrawn.new(m, File.dirname(a) ) 
						# @msg = output.msg if output.msg
						set_meta_data(output)
					end
				else 
					@msg = mailfile.msg
				end
				@log.info( trl("Processed %d eml-files") %num_files )
			end
			File.delete(tfile) if tfile && File.exist?(tfile)
			thr.stop("done")
		end
	end

	private

	# Should a user wish to either update her/his private version of the configuration-file by the current
	# contents of the file originally found in the downloaded program package, this function does the job.
	# It creates a copy of the original file, also, if there is not yet one, in ~/.cremefraiche/config or 
	# whatever the Configuration-singleton returns as file-name for user_configuration.
	def handle_user_conf
		conf_check = ConfCheck.new()
		diffs = conf_check.diffs
		replace = false
		if( diffs)
			if(diffs && diffs[:missing].empty? && diffs[:additional].empty?)
				puts trl("No changes to your user-configuration are necessary.")
				return false;
			else
				m = diffs[:missing]
				a = diffs[:additional]
				puts trl("\nThe following changes can be effectuated on your user-configuration")
				puts "\t+ " << trl("New options") << ":\n\t\t" << m.join('\n\t\t ') if !m.empty?
				puts "\t- " << trl("Obsolete Options") << ":\n\t\t" << a.join(', ') if !a.empty?
				puts trl("Do you want to replace your current user-configuration?")
				puts trl("If you answer 'yes' a back-up copy of your old file will still be available.")
				puts trl("But your old settings will be replaced by the defaults.")
				print trl("Your answer (Y/n): ")
				replace = STDIN.getc.downcase == trl("y") ? true : false
			end
			if(! replace)
				return false;
			end
		end
		conf_check.write_user_conf	
		return replace;
	end
=begin 
# moved to configuration

	# Finds external programs in the environment path.
	# A decision to use one of those programs depends on the fact, 
	# that they are installed at all. Contradicting options will 
	# result in warning-messages in the protocol output.
	def check_programs_installed
		prog_names = %w"pdftk evince acroread okular gpdf"
		$Progs = {} if !defined? $Progs
		pathdirs = ENV['PATH'].split(File::PATH_SEPARATOR).collect do |d| 
			# BUGFIX: PATH may contain just anything!
			pdir = nil
			if(Dir.exist?(d) )
				if(File.readable?(d) )
					# create a non nil entry for pathdirs.
					begin
						pdir = Dir.new(d) 
					rescue Exception => ex
						# "Trust nobody" (Herod V)
						@log.error(trl("Cannot search for executables in %s") %d << ': ' << ex.message )
					end
				else
					@log.warn(trl('The path-variable references an unreadable directory') << ': ' << d)
				end
			else
				@log.warn(trl('The path-variable contains an invalid value') << ': ' << d)
			end
			# --------> Now this is neat... 
			pdir
		end
		# ... or wait ... It is not!
		# Remove the nil-entries from pathdirs
		pathdirs.compact!
		# <------ Now this is neat... 

		if(pathdirs && !pathdirs.empty? )	
			@log.debug('path: ' + pathdirs.join(', ') )
			exts = ENV['PATHEXT'] ? ENV['PATHEXT'].split(';') : ['']

			prog_names.each do  |prog|
				pathdirs.each do |dir|
					exts.each do |ext|
						# BUGFIX: Beware of case-variations in the 
						# defined program-extension. Windows-specific. 
						# Thanks to Tijn Gommans for pointing this out.
						$Progs[prog] |= dir.entries.include?("#{prog}#{ext.upcase}")
						$Progs[prog] |= dir.entries.include?("#{prog}#{ext.downcase}")

					end
				end
			end
		end
		@log.debug("found the following external programs: " << $Progs.to_s)
	end
=end
	# Using PDFTK, this method will alter the values of some
	# meta-data fields in the resulting pdf-file.
	def set_meta_data(cfprawn)
		if($Progs['pdftk'] )
			encoding = 'iso-8859-1'
			cf = 'Cr&#232;me Fraiche'

			pdf = cfprawn.outfile
			headers = cfprawn.headers	
			t_file = "%s/temp_pdf"   %[$CONF.temp_dir.path]
			info_file = "%s/cr_info" %[$CONF.temp_dir.path]
			@log.debug('temp_dir path is ' << $CONF.temp_dir.path)
			meta_fields = %w"Author Subject Keywords Title"
			meta_data = {'Creator' => cf, 'Producer' => cf}
			meta_fields.each do |f|
				v = $CONF.send(f)
				meta_data[f] = v if v && !v.empty?
			end
			if(!meta_data['Subject'] )
				subject_header = headers.detect {|h| h.name == 'Subject'}
				meta_data['Subject'] = subject_header.value if subject_header
			end
			@log.debug('meta_data is ' << meta_data.inspect)
			begin
				File.open(info_file, 'w+') do |info|
					meta_data.each_pair do |k, v|
						info << @@Info_Key << k << "\n"
						info << @@Info_Value << v.encode(encoding) << "\n\n"
					end
				end
			rescue Exception =>ex
				@log.warn('PDFTK: ' << trl("Cannot write meta-data to temporary info-file (%s)" %ex.message))
			end
			cmd = Escape.shell_command(["pdftk", pdf, "update_info", info_file, "output", t_file]).to_s
			@log.debug('pdftk-command is ' << cmd)
			begin
				puts("\n" << trl("-----> The following output is created by pdftk. A PDF-file has already been created. <-----") )
				pdftk_result = system( cmd)
				puts("\n" << trl("<--------------------- End of pdftk output ------------------------------------------------>") )
				@log.debug('pdftk_result is: ' << pdftk_result.to_s)
				begin 
					FileUtils.mv(t_file, pdf)
				rescue Exception => ex
					@log.warn(trl("Cannot replace the pdf-file by a new version (%s)") %ex.message)
				end
			rescue Exception => ex
				@log.warn('PDFTK: ' << trl("Error upon execution of the command '%s': %s") %[cmd, ex.message] )
			end
		else
			@log.warn(trl('Metadata could not be written, pdftf is apparently not installed.'))
		end
	end

	# Currently unused.  Kept for reference.
	def result_message(results) 
		width = results.keys.any? ? results.keys.max {|a, b| a.length <=>b.length }.length : 0
		results.keys.each_with_index do |k, i|	
			puts "%i.) %#{width}s => %s" %[i+1, k, results[k] ? results[k] : trl("ERROR")] 
		end
	end

	# creates and prints the usage-message
	def self.usage
		CremeFraiche.new.usage
	end

	# Combines and returns a hash with information on 
	# the program and the system it is running on.
	def self.prog_info
		cf = CremeFraiche.new
		p_info = {:cur_system => cf.cur_system, :web_page => cf.web_page, :app_name => cf.app_name, :version => cf.version, :years => cf.years, :author => cf.author, :author_mail => cf.author_mail }
	end

	# prints the usage message
	def usage
		if $WIN
			system 'cls'
		else
			system( 'clear')
		end
		prog_version = (@app_name.dup << ' ' << trl("version %s") %[@version])
		sub_title = trl('EML to PDF converter')
		width = 32 + [prog_version, sub_title].max {|a, b| a.length <=>b.length}.length
		hr = '━' * width

		ustr = []
		ustr << " ┏%s┓" %hr
		ustr << " ┃%#{width - 16}s                ┃" %prog_version
		ustr << " ┃%#{width - 16}s                ┃" %sub_title
		ustr << " ┗%s┛" %hr
		ustr << ''
		ustr << " © %s %s %s" %[@years, @author, @author_mail] 
		ustr << ''
		ustr << " %s" %(trl("Running with Ruby %s on %s") %[RUBY_VERSION, RUBY_PLATFORM])
		ustr << ''
		u = trl("Usage")
		ulen = u.length + 1;
		ustr << " %s:" %u
		ustr << ('▔' * ulen )
		ustr << " \tuser@machine:~$ cremefraiche [%s.eml] <%s.eml ... %s(n).eml>" %[trl('eml_file1'), trl('eml_file2'), trl('eml_file')]
		ustr << ''
		ustr << " %s" %trl('This produces 1 PDF-file for each individual email found in the given eml-file(s).')
		ustr << " %s:" %trl("Please see the file 'config' in the program- or gem-folder for some options")
		ustr << " \t%s" %[$CONF.config_file]
		ustr << ''
		ustr << " %s:" %trl("To create or update a user-version of the configuration-file")
		ustr << ('▔' * ulen )
		ustr << " \tuser@machine:~$ ruby cremefraiche user-conf" 
		ustr << ''
		ustr << " %s %s" %[trl("Use literally 'user-conf' as argument."), trl("This either creates or overwrites a configuration file in a sub-directory '.cremefraiche' of the current user's home-directory")]
		ustr << "\n"
		ustr << "This program is free software under the terms of the GNU General Public License"
		ustr << "version 3 or any later version. It is provided as is with no guarantees of any kind."
		ustr << "\n" * 2
		ustr.join("\n")
	end
	public
	attr_reader :output, :web_page, :cur_system, :app_name, :version, :author, :years, :author_mail, :msg
end

# CremeFraiche.new(*ARGV)

