#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2011-2014 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative "file_checking"
require_relative "translating"
require_relative "logging"
require 'mail'


# A representation of an EML-file.
# Wraps the Mail-object (from the Mail-gem).
class EmlFile
	include File_Checking
	include Translating
	include Logging
	@@MAIL_INTRO = "From "
	@@errors = []
	attr_reader :headers, :body, :files, :filename, :msg
 
	public

	# create the representation of 1 eml-file
	def initialize(filename)
		init_logger($CONF.log_file ? $CONF.log_file : STDOUT, $CONF.log_level ? $CONF.log_level : Logger::UNKNOWN)
		@attachments = nil
		@body = nil
		@headers = nil
		msg = file_check(filename, :exist, :file, :readable)
		@log.debug('file checked: ' << msg.to_s)
		@@errors << msg if msg
		# split the file into many, one for each e-mail
		if(!msg)
			split(filename)
		else
			@log.error(trl("Invalid file") << ": %s (%s)" %[filename,msg] )
						
		end
	end
	# delegate messages, that this object cannot respond to to the Mail-object
	def method_missing(method, *args)
		if(@mail.respond_to?(method) )
			@mail.send(method, *args)
		else
			@log.error(trl("method %s is undefined for objects of class %s") %[method, self.class.name])
		end
	end
	# call the given block on each individual mail-file
	def mails(&b)
		if(@files)
			@log.debug('working on the files: ' << @files.join(', '))
			@files.each do |f|
				parse(f)
				@filename=f
				yield(self)
			end
		end
	end

	private

	# separate the components of each mail-file
	def parse(filename)
		if(filename)
			@mail = Mail.read(filename)
			if(@mail )
				@headers = @mail.header_fields
				# @log.debug('Parsed headers are ' << @headers.join(', ') )
				@body = @mail.body
				@log.debug('Parsed body is ' << @body.to_s )
				@attachments = @mail.attachments if @mail.attachment?
			end
		end
	end

	# prefix the number num with a maximum of count_zero zeros
	def prefix_number(num, count_zero = 2)
		str = ''
		if(num)
			str = num.to_s  
			count_zero.times do
				if(num % 10 == num)
					str = ('0' << str )
						@log.debug('str is now ' << str)
				end
			end	
		end
		return str
	end

	# split the file 'filename' into separate files, each containing 1 mail
	def split(filename)
		dir = $CONF.temp_dir
		@log.debug(trl("storing temporary files in %s") %dir.path)
		index = 0
		@files = []
		begin
			File.open(filename, 'r') do |ifl|
				ofl = nil
				ifl.each_line do |line|
					if(line.start_with?(@@MAIL_INTRO))	
						ofl.close if ofl && !ofl.closed?
						index += 1
						ofl_name = format("%s/%s_%s" %[dir.path, prefix_number(index, 2), File.basename(filename)])
						@log.debug(trl("storing a message in %s") %ofl_name) 
						ofl = File.open(ofl_name, 'w')
						@files << ofl_name
					end
					ofl << line if ofl && !ofl.closed? 
				end
				ofl.close if ofl && !ofl.closed?
			end
		rescue Exception => ex
			@log.error(trl("Cannot open file '%s'") %filename << ': ' <<  ex.message )
		end
		# a single e-mail can be stored in its own file
		if(@files.empty? && File::exist?(filename) && File::readable?(filename) )
			@files << filename  
		end
	end # END split()
end # END of class

