#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2012-2016, Michael Uplawski <michael.uplawski@uplawski.eu>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'gtk3'
require_relative '../cremefraiche'
require_relative 'ButtonLabel'
require_relative '../translating'
require_relative '../file_checking'
require_relative '../logging'


class HowtoDialog
	include Translating
	include Logging
	@@GD = File::expand_path(File::dirname(__FILE__) ) + File::Separator if !defined?(@@GD)
	@@HOWTO = $PROG_DIR.dup << '../doc/gui_howto.html'
	@@NAVIGATORS = %w"firefox opera epiphany-browser"

	def initialize(options = {})
		init_logger($CONF.log_file ? $CONF.log_file : STDOUT, $CONF.log_level ? $CONF.log_level : Logger::UNKNOWN)
		@hdialog = Gtk::Dialog.new()
		reader_frame = Gtk::Frame.new(trl('Howto'))

		reader_vbox = Gtk::Box.new(:vertical, 5)
		reader_vbox.border_width = 5
		reader_hbox = Gtk::Box.new(:horizontal, 5)
		reader_hbox.border_width = 5

		bl_text = trl("Please name the command which starts your Web-Browser (e.g. firefox, opera, epiphany-browser ...).")
		reader_label = Gtk::Label.new(bl_text)
		reader_label.wrap = true
		@reader_entry = Gtk::Entry.new                                 
		@reader_entry.width_chars = 33
		@reader_entry.height_request = 25
		# Sabotage fix. “We, the Gods of GTK approve of this failure”
		# @reader_entry.text = $CONF.pdf_reader
		@reader_entry.text = find_reader()
		@bc_button = Gtk::Button.new()
		@bc_button.add(ButtonLabel.new(trl('_Choose'), "%s/gtk-open.xpm" %@@GD))

		reader_vbox.pack_start(reader_label, :expand => false)

		reader_hbox.pack_start(@reader_entry, :expand => false)
		reader_hbox.pack_start(@bc_button, :expand => false)
		reader_vbox.pack_start(reader_hbox, :expand => false)
		reader_frame.add(reader_vbox)

		button_box = Gtk::ButtonBox.new(:horizontal)
		button_box.layout_style = :start
		@cancel_button = Gtk::Button.new()
		@cancel_button.add(ButtonLabel.new(trl('_Cancel'), "%s/gtk-stop.xpm" %@@GD))
		@ok_button = Gtk::Button.new()
		@ok_button.add(ButtonLabel.new(trl('_Accept'), "%s/gtk-execute.xpm" %@@GD))

		button_box.pack_start(@ok_button, :expand => false)
		button_box.pack_start(@cancel_button, :expand => false)

		@hdialog.child.pack_start(reader_frame)
		@hdialog.child.pack_start(button_box)

		@bc_button.set_tooltip_text trl('Choose a Web-Browser')
		@reader_entry.set_tooltip_text trl('Command, which starts your Web-Browser')
		@cancel_button.set_tooltip_text trl('Close dialog')
		@ok_button.set_tooltip_text trl('Executes your Web-Browser and shows the Howto manual')

		connect_buttons
		@hdialog.set_title('HOWTO: ' << CremeFraiche::prog_info[:app_name])
		@hdialog.child.show_all
		@hdialog.signal_connect('response') do |dlg, resp|
			@log.debug("response %i emitted" %resp) if @log
			@adialog.destroy if(:cancel == resp) 
		end  
		if(options && options.keys.include?(:on_destroy))
			@hdialog.signal_connect('destroy') {options[:on_destroy].call() }
		end

		@hdialog.show
	end

	def find_reader
		@@NAVIGATORS.detect(''){|c| $Progs.include?(c)}
	end

	def choose_reader
		dialog = Gtk::FileChooserDialog.new(:title => trl("set reader"),
						    :parent => nil,
						    :action => :open,
						    :buttons => [[:cancel, :cancel], [:accept, :accept]])

		if(@reader_entry.text && ! @reader_entry.text.empty?)
			@log.debug('reader is ' << @reader_entry.text)
			dialog.filename = @reader_entry.text 
		end
		ret = dialog.run
		br = (ret == Gtk::ResponseType::ACCEPT ? dialog.filename : nil)
		dialog.destroy

		msg = File_Checking.file_check(br, [:exist?, :file?, :executable?]) if br   
		if(msg)
			@log.error(msg) if @log
			msg_dialog(msg, DLG_ERROR) 
			br = nil
		end
		return br
	end

	def connect_buttons
		@bc_button.signal_connect("clicked") do 
			br = choose_reader
			@reader_entry.text = br if br
		end

		@cancel_button.signal_connect("clicked") do
			@hdialog.destroy
		end
		@ok_button.signal_connect("clicked") do
			run_reader(@reader_entry.text) if @reader_entry.text
		end
	end

	def run_reader(br)
		msg = nil
		if(br && ! br.empty?)
			begin
				fork_sym = (RUBY_PLATFORM =~ /win/) ? '' : ' &'
				exec_command = "%s %s%s" %[br, @@HOWTO, fork_sym]
				@log.debug "exec-command will be %s" %exec_command if @log
				hr = system exec_command
				if(!hr)
					msg = trl("Cannot open the Web-Browser!", "\n") 
					msg += trl("Please verify, that %s is correctly spelled or use another reader.", "\"#{br}\"") 
				else
					@hdialog.destroy
				end
			rescue Exception => ex
				msg = trl("Cannot open the Web-Browser %s", ex)
			end
		else
			msg = trl('No reader selected, skipping Howto.')
		end
		@log.error(msg) if @log && msg
		@log.debug($?.to_s) if @log
		msg_dialog(msg, DLG_ERROR) if msg
	end

	def msg_dialog(msg, type)
		dialog = Gtk::MessageDialog.new(
						:parent => nil, 
						:flags => :destroy_with_parent,
						:type => type,
						:message => "%s: \"%s\"" %[Translating::trl(type.to_s.upcase), msg])

		dialog.set_title(DLG_INFO == type ? 'Info' : 'Error' )
		dialog.run
		dialog.destroy
		return msg
	end


end

# execute if this script is the argument to the ruby-interpreter.
if __FILE__ == $0
	HowtoDialog.new()
end

