#!/usr/local/bin/env ruby
#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2012-2016, Michael Uplawski <michael.uplawski@uplawski.eu>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative '../logging'
require_relative '../translating'
require_relative 'message_dialog'
require_relative '../cremefraiche'

CHECK_GEMS = false

if(CHECK_GEMS)
	['nokogiri', 'prawn', 'mail', 'gtk3', 'escape'].each do |g|
		begin
			gem g
		rescue LoadError
			missing_gems = true
			puts '   =>   ' << (Translating::trl("PSE wait, installing required gem '%s'") %g)
			puts '        ' << Translating::trl('This may take some time') << ' ...'
			begin	
				if  !system 'gem install ' << g
					exit false
				end
			rescue Exception => e
				puts '        ' << (Translating::trl("Cannot install '%s'") %g) << ': ' << e.message
				exit false
			end
		end
		Gem.clear_paths
	end
end

require 'gtk3'
require 'ostruct'
require_relative '../file_checking'
require_relative 'ConfDialog'
require_relative 'HowtoDialog'
require_relative 'AboutDialog'
require_relative 'ButtonLabel'


DLG_ERROR = :error
DLG_INFO = :info
DLG_WARN = :warning

# maximum length of the file-names before they are clipped
MAX_FN_LEN = 35

HOME = ENV['HOME'].dup

# the list of eml-files is stored each time upon quitting
LIST_FILE = HOME << File::Separator << '.cremefraiche.lst'

# String-constants
CLICKED = 'clicked'
LN = "\n"
FIELD_SEPARATOR = '#'
LINE_SEPARATOR = nil

=begin
 A simple user-interface for Crème Fraiche.
=end

class CremeFraicheGui
	include Translating
	include Logging
	@@GD = File::expand_path(File::dirname(__FILE__) ) + File::Separator if !defined?(@@GD)
	def initialize(args = nil)
		init_logger( ($CONF.log_file ? $CONF.log_file : STDOUT), ($CONF.log_level ? $CONF.log_level : Logger::UNKNOWN) )
                @log.info('$CONF is ' << $CONF.to_s)
                @log.info('@log.level is ' << @log.level.to_s)
		@log.debug('init gui')
		prog_info = CremeFraiche::prog_info()
		@eml_files = {}

		@window = Gtk::Window.new(prog_info[:app_name])
		@window.set_default_size(300, 100)
		@window.set_icon($ICON_DIR + 'icon.xpm');
		# populate the window
		create
		# prepare an object to collect the tooltips
		@tooltips = Gtk::Tooltip.new
		# create some static tool-tips for the static elements
		set_tooltips
		# connect the static buttons with the functionality
		connect_buttons
		# the transformations in progress	
		@threads = Array.new
		# restore list of files from previous session
		read_list
		# make sure, that there will be only one file-dialog at a time.
		@fdialog = nil
		# display on screen
		show
	end
	private

	def show
		@window.show
		Gtk::main
	end

	def create
		# layout-elements
		top_vbox = Gtk::Box.new(:vertical, 5)
		top_vbox.border_width = 5 

		eml_vbox = Gtk::Box.new(:vertical, 2)

		eml_hbox = Gtk::Box.new(:horizontal, 5)
		eml_hbox.border_width = 5

		# the text-entry for the chosen file-names
		@eml_input = Gtk::Entry.new
		@eml_input.width_chars = 33
		@eml_input.height_request = 25
		# the button to add a file-name to the list
		@eml_add_button = Gtk::Button.new
		@eml_add_button.add(ButtonLabel.new( trl('Add _file') ))
		@eml_add_button.height_request = 0.0 
		# a button to open a file-chooser
		@eml_button = Gtk::Button.new()
		@eml_button.add(ButtonLabel.new(trl('_Choose'), @@GD + "gtk-open.xpm"))
		@eml_button.height_request = 0.0 
		# organize the above elements	
		eml_hbox.pack_start(@eml_button, :expand => false, :fill => false)
		eml_hbox.pack_start(@eml_input, :expand => true, :fill => true)
		eml_hbox.pack_end(@eml_add_button, :expand => false, :fill => false)
		# a group of horizontally aligned buttons
		button_box = Gtk::ButtonBox.new(:horizontal)
		button_box.layout_style = Gtk::ButtonBoxStyle::START

		@quit_button = Gtk::Button.new()
		@quit_button.add(ButtonLabel.new(trl('_Quit'), @@GD + "gtk-quit.xpm" ))
		@start_button = Gtk::Button.new()
		@start_button.add(ButtonLabel.new(trl('Sta_rt'), @@GD + "gtk-execute.xpm"))

		@stop_button = Gtk::Button.new()
		@stop_button.add(ButtonLabel.new(trl('Sto_p'), @@GD + "gtk-stop.xpm") )
=begin
		@save_button = Gtk::Button.new()
		@save_button.add(ButtonLabel.new(trl('_Save'), @@GD + "gtk-save.xpm") )
=end
		@config_button = Gtk::Button.new()
		@config_button.add(ButtonLabel.new(trl('_Configure'), @@GD + "gtk-properties.xpm" ))

		@about_button = Gtk::Button.new()
		@about_button.add(ButtonLabel.new(trl('_About'), @@GD + "gtk-about.xpm"))

		@howto_button = Gtk::Button.new()
		@howto_button.add(ButtonLabel.new(trl('_Howto'), @@GD + "help.xpm"))

		button_box.pack_start(@start_button, :expand => false, :fill => false)
		button_box.pack_start(@stop_button, :expand => false, :fill => false)
		# button_box.pack_start(@save_button, false)
		button_box.pack_start(@quit_button, :expand => false, :fill => false)
		button_box.pack_start(@config_button, :expand => false, :fill => false)
		button_box.pack_start(@about_button, :expand => false, :fill => false)
		button_box.pack_start(@howto_button, :expand => false, :fill => false)
		# a table to show chosen file-names and some controls
		@table = Gtk::Table.new(1,5)
		tframe = Gtk::Frame.new()
		tframe.add(@table)
		# organize all of the above created elements on the window
		eml_vbox.pack_start(eml_hbox, :expand => false)
		eml_vbox.pack_end(tframe, :expand => false)


		@status_label = Gtk::Label.new()

		top_vbox.pack_start(eml_vbox)
		top_vbox.pack_end(@status_label)
		top_vbox.pack_end(button_box)
		# initialize the table
		refresh_table
		# display all
		top_vbox.show_all
		@window.add(top_vbox)
		@mthr = nil
	end

=begin
	creates and allocates some static tool-tips
=end
	def set_tooltips
		@eml_button.set_tooltip_text(trl('Choose an eml-file to transform') )
		@eml_input.set_tooltip_text trl('Enter or choose an eml-file to transform')
		@eml_add_button.set_tooltip_text trl('First choose an eml-file, then add it to the list')
		@quit_button.set_tooltip_text trl('Quit program')
		@start_button.set_tooltip_text trl('Transform the active files')
		@stop_button.set_tooltip_text trl('Terminate all transformations in progress')

		@about_button.set_tooltip_text trl('About this program')
		@howto_button.set_tooltip_text trl('Howto Crème Fraiche GUI')
	end
        
=begin
  unused for the time.
=end
        def review_configuration()

        end
=begin
	connect the existing buttons to their functionality	
=end
	def connect_buttons
		@eml_button.signal_connect(CLICKED) do
			eml = choose_eml
			if(eml)
				@eml_input.text = eml 
			else
				status_message(trl('No file chosen'), 2)
			end
		end
		@eml_add_button.signal_connect(CLICKED) do
			if @eml_input.text && !@eml_input.text.empty?
				add_eml(@eml_input.text)
			else
				status_message(trl('No file chosen, none added'), 2 )
			end
		end
		@quit_button.signal_connect(CLICKED) do
			stop()
			save()
			Gtk::main_quit()
		end
		@window.signal_connect("destroy") do
			stop()
			save()
			Gtk.main_quit
		end
		@about_button.signal_connect(CLICKED) do
			@about_button.sensitive=false
			db = Proc.new() { @about_button.sensitive=true }
			ad = AboutDialog.new(:on_destroy => db)
		end
		@config_button.signal_connect(CLICKED) do
			@config_button.sensitive=false
                        cb = Proc.new() do 
                                @config_button.sensitive=true
                                review_configuration()
                        end
                        cd = ConfDialog.new(:on_destroy => cb)
                end
                @howto_button.signal_connect(CLICKED) do
                        @howto_button.sensitive=false
                        rb = Proc.new() { @howto_button.sensitive=true }
                        hd = HowtoDialog.new(:on_destroy => rb)
                end
                @start_button.signal_connect(CLICKED) do
                        @start_button.sensitive=false
                        @eml_files.each_value {|ctx| ctx.ok.set_file(nil)}
                        transform()
                end
                @stop_button.signal_connect(CLICKED) do
                        stop()
                end
=begin
                @save_button.signal_connect(CLICKED) do
                        save()
                end
=end
        end

=begin
        terminate all running transformations
=end
        def stop( pause = 2)
                if(@threads && !@threads.empty?)	
                        @threads.each do |th| 
                                th.terminate
                                th.join
                        end
                        refresh_table
                        status_message(trl('transformation stopped'), 2)
                        @start_button.sensitive=true
                else
                        status_message(trl('No transformations in progress'), 2)
                end
        end
=begin
        restore the list of eml-files to transform from a previously saved
        file.
=end
        def read_list
                # BUGFIX: make sure, that the list-file exists already
                lf = File.open(LIST_FILE, 'a+')
                lf.close
                # END BUGFIX 
                # BUGFIX: do not split into arbitrary chunks of text, DUMBO!
                list = File.readlines(LIST_FILE)
                entry = []
                list.each do |line| 
                        entry = line.chomp.split(FIELD_SEPARATOR)
                        add_eml(entry[0], entry[1] == true.to_s)
                end	
                #end BUGFIX
        end

=begin
        Save the list of files for later transformations, overwrite
        previous file-content.
=end
        def save() 
                list = [] 
                @eml_files.each_pair do |eml, ctx|
                        # Ensure, that there is not more than one FIELD_SEPARATOR in the list-entry.
                        list << (eml.dup.gsub(FIELD_SEPARATOR, '*') << FIELD_SEPARATOR << ctx.active.to_s)
                end
                File.write(LIST_FILE, list.join(LN))
        end
=begin
        re-construct the table from the list of chosen file-names
        or show a message, if no files have ben chosen yet.
=end
        def refresh_table 
                # remove all contents from the current table
                @table.each {|w| @table.remove(w) }
                if(!@eml_files.empty?)
                        # create a table header in the cells of the first row
                        [trl('active'), trl('Ok'), trl('file'), trl('size'), trl('remove')].each_with_index do |t, i|
                                label = Gtk::Label.new
                                label.set_markup("<b>#{t}</b>")
                                @table.attach(label, i, i + 1, 0, 1)
                        end
                        # fill the remainder of the table with file-names and 
                        # control-elements
                        @eml_files.each_pair do |file, ctx|
                                text = file.dup
                                filename = File::basename(file)
                                # shorten file-names, if need be
                                if text.length > MAX_FN_LEN + 3
                                        num_del = text.length - MAX_FN_LEN - 3	
                                        text = "..." + text[num_del, text.length]
                                end
                                fl = Gtk::Label.new(text)
                                fl.set_alignment( 1, 0)
                                # store the reference to the label
                                ctx.label = fl
                                # show the full file-path in the tool-tip to the text-label
                                fl.set_tooltip_text(file)
                                # a check-button to activate the transformation of the very file
                                cb = Gtk::CheckButton.new()
                                # set checkbox active or not
                                cb.set_active(ctx.active)
                                cb.set_tooltip_text trl("Controls whether the file '%s' is transformed or not.") %filename
                                # an image which will indicate success
                                ctx.ok = Gtk::Image.new()
                                # a button to remove the file from the list
                                rb = Gtk::Button.new
                                rb.height_request = 5
                                rb.add(ButtonLabel.new('', @@GD + "gtk-remove.xpm" ))
                                rb.set_tooltip_text trl("Remove the file '%s' from the list") %filename
                                begin
                                        fs = Gtk::Label.new(readable_size(File::size(file)) )
                                        fs.set_alignment(0.8,0) 
                                        fs.set_tooltip_text trl("Size of %s") %filename
                                        # add the above elements to the table
                                        @table.n_rows = @table.n_rows + 1
                                        @table.attach(cb, 0, 1, @table.n_rows - 1, @table.n_rows, :shrink )
                                        @table.attach(ctx.ok, 1, 2, @table.n_rows - 1, @table.n_rows )
                                        @table.attach(fl, 2, 3, @table.n_rows - 1, @table.n_rows )
                                        @table.attach(fs, 3, 4, @table.n_rows - 1, @table.n_rows )
                                        @table.attach(rb, 4, 5, @table.n_rows - 1, @table.n_rows, :shrink )
                                        # connect the 'remove' button to its functionality
                                        rb.signal_connect(CLICKED) do 
                                                del_eml(file)
                                        end
                                        cb.signal_connect(CLICKED) do
                                                toggle_active(file)
                                        end
                                rescue Exception => ex
                                        msg = trl("File %s cannot be handled! (%s)" %[file, ex.message] )
                                end
                        end
                else
                        # no files have been chosen yet, report this fact.
                        mlabel = Gtk::Label.new()
                        mlabel.set_markup("<span background='#f0a0a0'>#{trl('No files have been chosen for transformation') }</span>" )
                        @table.attach(mlabel, 0, 3, 0, 1)
                end
                # display the table
                @table.show_all
        end
=begin
        excempt a file from the transformation, but leave it in the list.
=end
        def toggle_active(file = nil)
                if(file)
                        isso = @eml_files[file].active 
                        @eml_files[file].active = (isso ? false : true)
                end
        end
=begin
        remove a file from the list of files that can be transformed.
=end
        def del_eml(file = nil)
                if(file )
                        @eml_files.delete_if {|f, context| f == file }
                        refresh_table
                        status_message(trl('File removed'), 2 )
                end
        end
=begin
        add a file to the list of files that can be transformed
=end
        def add_eml(file = nil, activate = true)
                if(file )
                        if( !@eml_files.keys.include?(file) )
                                msg = File_Checking::file_check(file, :exist?, :file?, :readable?)
                                if !msg
                                        file_ctx = OpenStruct.new
                                        file_ctx.active = activate
                                        @eml_files[file] = file_ctx
                                        refresh_table
                                        status_message(trl('File added'), 2)
                                else
                                        msg = trl("Cannot add the file %s (%s)") %[file, msg]
                                        @log.warn(msg)
                                        msg_dialog(msg, DLG_ERROR) 
                                end
                        else
                                status_message(trl('File is already in the list'))
                        end
                end
        end
=begin
        open a file-chooser and allow the selection of a file for transformation.
=end
        def choose_eml
                # only if the dialog is not currently open
                if(!@fdialog)
                        @fdialog = Gtk::FileChooserDialog.new(:title => trl("Choose EML-File"),
                                                              :parent => nil,
                                                              :action => :open,
                                                              :buttons => [[:cancel, :cancel],
                                                                           [:open, :accept]])
                        if(@eml_input.text && ! @eml_input.text.empty?)
                                # start where the currently selected file has been located.
                                @fdialog.filename = @eml_input.text 
                        else
                                # or start in the user's home-directory
                                @fdialog.filename = HOME 
                                @fdialog.filename ||= Dir::pwd
                        end
                        # open the file-chooser, return the selected file-name
                        eml = (@fdialog.run == Gtk::ResponseType::ACCEPT ? @fdialog.filename : nil)
                        @fdialog.destroy
                        # verify that the file is useable
                        msg = File_Checking.file_check(eml, :exist?, :file?, :readable?)      
                        if(msg)
                                @log.error(msg) if @log
                                msg_dialog(msg, DLG_ERROR) 
                                eml = nil
                        end
                        @eml_add_button.set_tooltip_text( trl('Add file to the list of files to transform')) if eml
                        @fdialog = nil
                        # return the file-name
                        return eml
                end
        end

=begin
        # Any message, related to the GUI and the user-input can be
        # displayed. The second parameter determines, if it is
        # an information or an error and makes the dialog choose its
        # icon accordingly.
        def msg_dialog(msg, type)
                CremeFraicheGui::msg_dialog(msg, type)
        end

        def self::msg_dialog(msg, type)
                nick = type && type.respond_to?(:nick) ? type.nick : type.to_s
                msg_dialog(type, msg)	
                dialog = Gtk::MessageDialog.new(:parent => nil, 
                                                :flags => 0,
                                                :destroy_with_parent,
                                                :type => type,
                                                :buttons => :close,
                                                "%s: \"%s\"" %[Translating::trl(nick.to_s), msg])
                dialog.set_title(DLG_INFO == type ? 'Info' : 'Error' )
                dialog.run
                dialog.destroy
                return msg
        end
=end

        def status_wait(wait_message )
                @mthr.terminate if @mthr	
                @mthr.join if @mthr
                @mthr = Thread.new do
                        loop do
                                red = rand(0xe0...0xff)
                                green = rand(0xe0...0xff)
                                blue = 0xc0
                                @status_label.set_markup("<span background='#%02x%02x%02x'>#{wait_message}</span>" %[red, green, blue] )
                                sleep 0.3 
                        end
                end
        end
        def status_message(msg, time = 2)
                @mthr.terminate if @mthr	
                @mthr.join if @mthr
                @mthr = Thread.new do
                        @status_label.text = msg
                        if time && time > 0
                                sleep time 
                                @status_label.text = ''
                        end
                end
        end

        def transform()
                if !@eml_files.empty?
                        status_message(trl("Transforming, wait..."), nil)
                        @threads.clear if @threads && !@threads.empty?
                        active_files = @eml_files.values.count {|ctx| ctx.active}
                        if(active_files > 0)
                                @eml_files.each_pair do |file, ctx|
                                        if(ctx.active)
                                                ctx.label.set_markup(file)
                                                @threads << Thread.new do 
                                                        wait_th = status_wait(trl("Transforming, wait..."))
                                                        cf = CremeFraiche.new(file) 
                                                        if(cf.msg && !cf.msg.empty?)
                                                                msg_dialog(cf.msg, DLG_ERROR)
                                                        end

                                                        wait_th.terminate
                                                        wait_th = nil
                                                        active_files -= 1;
                                                        ctx.label.set_markup("<span background='#c0f0c0'><b>#{file}</b></span>")
                                                        ctx.ok.set_file(@@GD + 'okay.xpm')
                                                        #status_message( trl("file %s has been transformed.") %File::basename(file), 2)
                                                        if(active_files == 0)
                                                                status_message( trl("All done"), 5 )
                                                                @threads.clear
                                                                @start_button.sensitive=true
                                                                @eml_files.values.each{|ctx| ctx.label.set_markup(ctx.label.text) }
                                                        end
                                                end
                                        end
                                        @table.show_all	
                                end
                        else
                                status_message(trl('No files have been selected for transformation'), 2)
                                @start_button.sensitive=true
                        end
                else
                        status_message(trl('Empty file-list! Doing nothing!'), 2)
                        @start_button.sensitive=true
                end
        end

        def run_reader(br, file = nil)
                msg = nil
                if(br && ! br.empty?)
                        begin
                                fork_sym = (RUBY_PLATFORM =~ /win/) ? '' : ' &'
                                if(file)
                                        exec_command = "%s %s%s" %[br, file, fork_sym]
                                else
                                        exec_command = "%s %s" %[br, fork_sym]
                                end
                                @log.debug "exec-command will be %s" %exec_command if @log
                                hr = system exec_command
                                if(!hr)
                                        msg = trl("Cannot open the PDF-reader!") << "\n" 
                                        msg << trl("Please verify, that %s is correctly spelled or use another reader.", %("\" << br << "\"") ) 
                                else
                                        @hdialog.destroy
                                end
                        rescue Exception => ex
                                msg = trl("Cannot open the PDF-reader XX", ex)
                        end
                else
                        msg = trl('No reader selected, skipping Howto.')
                end
                @log.error(msg) if @log && msg
                @log.debug($?.to_s) if @log
                msg_dialog(msg, DLG_ERROR) if msg
        end


        def readable_size(size)
                n_size = size
                symbol = 'b'
                if(size && size.respond_to?(:to_f)) 
                        if(size >= 1000000000)	
                                n_size = size.to_f/1000000000 
                                symbol = 'G'
                        elsif(size >= 1000000)	
                                n_size = size.to_f/1000000 
                                symbol = 'M'
                        elsif(size >= 1000)
                                n_size = size.to_f/1000 
                                symbol = 'K'
                        end
                end
                return n_size.round(1).to_s << symbol
        end
end

# execute if this script is the argument to the ruby-interpreter.
# CremeFraicheGui.new(ARGV)

