#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2011-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'rubygems'
require 'nokogiri'
require_relative 'logging'
require_relative 'configuration'
require_relative 'tag_munging'
# An object of this class formats the HTML from an email for rendering
# in a PRAWN-generated PDF-file

class Html2Text 
	include Logging
	include TagMunging
	def initialize( html)
		init_logger($CONF.log_file ? $CONF.log_file : STDOUT, $CONF.log_level ? $CONF.log_level : Logger::UNKNOWN)
		#remove line-breaks
		html.gsub!("\n", ' ')
		# whatever...
		html.gsub!("<BR>", "\n")
		html.gsub!("<BR/>", "\n")
		html.gsub!("<br>", "\n")
		html.gsub!("<br/>", "\n")
		html.gsub!("<br />", "\n")
		#remove html-comments
		html.gsub!(/\<\!--([\s*\S*])*-->/, '')
		@html = Nokogiri::HTML::fragment(html)
		define_tag_handlers if @html
		@out_text = ''
	end

	# def text(tag = @html)
	def text()
		@out_text = handle(@html)
	end

	#TODO: verify children in the node-list
	def handle(tag = @html)
		@log.debug('handle tag ' << tag.name)
		tcontent = ''
		if(tag.respond_to?(:children) )
			tag.children().each do |t|
				@log.debug("\tchild tag " << t.name)
				tresult = nil
				handler = @@tag_handlers[t.name.downcase]	
				if(!handler)
					handler = @@tag_handlers['any']
				end
				tresult = handler.call(t)
				tcontent << (tresult ? tresult : '')
			end
		else
			tcontent = tag.to_s
		end
		tcontent
	end

	private
	def define_tag_handlers
		@@tag_handlers = {}

		do_nothing = Proc.new() do |value|
			handle(value)
		end

		@@tag_handlers['a'] = Proc.new() do |value|
			v = handle(value)
			link_color = "#" << $CONF.link_color if $CONF.link_color 
			link_color ||= DEF_LINK_COLOR

			# someting in Nokogiri prevents me from setting the link content,
			# I write the tag explicitly.
			link = '<link href="' << value['href'] << '">'
			link << '<color rgb="' << link_color << '">'
			link << v << '</color></link>'
=begin
# DOES NOT WORK:
			link = new_tag(@html, 'link', :content => v, :href => value['href'])
			color = new_tag(@html, 'color', :parent => link)	
			color['rgb'] = link_color
=end
			@log.debug('link is : ' << link.to_s)
			link
		end

		@@tag_handlers['strong'] = Proc.new() do |value|
			"<b>" << handle(value) <<"</b>"
		end
		@@tag_handlers['body'] = Proc.new() do |value|
			handle(value)
		end
		@@tag_handlers['blockquote'] = Proc.new() do |value|
			content = "\n ------------------------------- "
			handle(value).split("\n").each do |l|
				content << "\n| " << l
			end
			content << "\n ------------------------------- "
			content
		end

		@@tag_handlers['div'] = Proc.new() do |value|
			"\n" << handle(value)
		end

		@@tag_handlers['syle'] = do_nothing
		@@tag_handlers['meta'] = do_nothing
		@@tag_handlers['head'] = do_nothing
		@@tag_handlers['tbody'] = do_nothing
		@@tag_handlers['doctype'] = do_nothing
		@@tag_handlers['font'] = do_nothing
		@@tag_handlers['img'] = do_nothing

		@@tag_handlers['span'] = Proc.new() do |value|
			handle(value)
		end	


		@@tag_handlers['title'] = Proc.new() do |value|
			"<b>" << handle(value ) << "</b>"
		end

		@@tag_handlers['td'] = Proc.new() do |value|
			@log.debug("\t\thandler td : " << value)
			v = handle(value)
			ntag = ''
			if(v && !v.strip.empty?)
				ntag << v
			end
			ntag
		end
		@@tag_handlers['table'] = Proc.new() do |value|
			handle(value)
		end

		@@tag_handlers['tr'] = Proc.new() do |value|
			@log.debug("\t\thandler tr : " << value)
			"\n" << handle(value )
		end


		@@tag_handlers['p'] = Proc.new() do |value|
			"\n" << handle(value )
		end

		@@tag_handlers['b'] = Proc.new() do |value|
			'<b>' << handle(value) << ' </b>' 
		end	

		@@tag_handlers['i'] = Proc.new() do |value| 
			"<i>" << handle(value)<< "</i>" 
		end	

		@@tag_handlers['any'] = Proc.new() do |value|
			@log.debug("\t\thandling any : " << value)
			value
		end
	end
end

