#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2013-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative 'logging'
require_relative 'translating'
require 'nokogiri'
require 'singleton'

=begin
	A utility module for recurring tag-specific tasks.
=end
module TagMunging

	def remove_empty_rows(table)
		all_body_rows(table) do |tr| 
			cells = tr.xpath("./td").dup
			empty = true
			cells.each{|cell| empty = false if !cell.children.empty? }	
			tr.remove if empty
		end
	end

	def all_row_cells(tr, &b)
		tds = tr.xpath('./td')
		tds.each do |td|
			yield(td, tds.index(td) )
		end
	end

	def all_body_rows(table, &b)
		trs = table.xpath("./tbody/tr")
		@log.debug('table contains ' << trs.size.to_s << ' data-rows') if @log
		trs.each do |tr|
			yield(tr, trs.index(tr) )
		end
	end

	def all_body_cells(table, &b)
		trs = table.xpath("./tbody/tr")
		@log.debug('table contains ' << trs.size.to_s << ' data-rows') if @log
		trs.each do |tr|
			all_row_cells(tr) do |td|
				yield(td, trs.index(td) )
			end
		end
	end

	def remove_empty_sub_tags(parent, tag)
		tags = parent.xpath(".//" << tag)
		tags.each{|t| t.remove if t.content.chomp.strip.empty?}
	end

	def right_align_numbers(tags, symbol)
		tags.each do |tag|
			if(tag.content.chomp.match(/^-?[^\D]\d*(\.?\d*)?[^\D]?$/) )
				tag['class'] = (tag['class'] ? tag['class'] : '') << ' ' << 'right'	
				cont = tag.content
				tag.content = cont << symbol if cont && symbol
			end
		end
	end

	def new_tag(doc, tag, options = nil)
		ntag = Nokogiri::XML::Node.new(tag, doc)
		if(options && options.respond_to?(:to_hash))
			ntag.content = options.delete(:content) if options[:content]
			ntag.parent = options.delete(:parent) if options[:parent]	
			options.each do |k,v|
				ntag[k.to_s] = v 
			end
		end
		return ntag
	end

	def tag_by_content(doc, tagname, content)
		return doc.xpath("//" << tagname.to_s).detect{|tag| tag.content.strip == content}	
	end
end
